package javax.swing.tree;

import java.util.NoSuchElementException;
import sun.swing.SwingUtilities2;
import javax.swing.event.TreeModelEvent;
import java.util.Enumeration;
import java.util.Stack;
import java.util.Hashtable;
import java.awt.Rectangle;

public class FixedHeightLayoutCache extends AbstractLayoutCache
{
    private FixedHeightLayoutCache.FHTreeStateNode root;
    private int rowCount;
    private Rectangle boundsBuffer;
    private Hashtable<TreePath, FixedHeightLayoutCache.FHTreeStateNode> treePathMapping;
    private FixedHeightLayoutCache.SearchInfo info;
    private Stack<Stack<TreePath>> tempStacks;
    
    public FixedHeightLayoutCache() {
        this.tempStacks = new Stack();
        this.boundsBuffer = new Rectangle();
        this.treePathMapping = new Hashtable();
        this.info = new FixedHeightLayoutCache.SearchInfo();
        this.setRowHeight(1);
    }
    
    @Override
    public void setModel(TreeModel model) {
        super.setModel(model);
        this.rebuild(false);
    }
    
    @Override
    public void setRootVisible(boolean rootVisible) {
        if (this.isRootVisible() != rootVisible) {
            super.setRootVisible(rootVisible);
            if (this.root != null) {
                if (rootVisible) {
                    ++this.rowCount;
                    this.root.adjustRowBy(1);
                }
                else {
                    --this.rowCount;
                    this.root.adjustRowBy(-1);
                }
                this.visibleNodesChanged();
            }
        }
    }
    
    @Override
    public void setRowHeight(int rowHeight) {
        if (rowHeight <= 0) {
            throw new IllegalArgumentException("FixedHeightLayoutCache only supports row heights greater than 0");
        }
        if (this.getRowHeight() != rowHeight) {
            super.setRowHeight(rowHeight);
            this.visibleNodesChanged();
        }
        return;
    }
    
    @Override
    public int getRowCount() {
        return this.rowCount;
    }
    
    @Override
    public void invalidatePathBounds(TreePath treePath) {
    }
    
    @Override
    public void invalidateSizes() {
        this.visibleNodesChanged();
    }
    
    @Override
    public boolean isExpanded(TreePath treePath) {
        if (treePath != null) {
            FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
            return nodeForPath != null && nodeForPath.isExpanded();
        }
        return false;
    }
    
    @Override
    public Rectangle getBounds(TreePath treePath, Rectangle rectangle) {
        if (treePath == null) {
            return null;
        }
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            return this.getBounds(nodeForPath, -1, rectangle);
        }
        TreePath parentPath = treePath.getParentPath();
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath2 = this.getNodeForPath(parentPath, true, false);
        if (nodeForPath2 != null && nodeForPath2.isExpanded()) {
            int indexOfChild = this.treeModel.getIndexOfChild(parentPath.getLastPathComponent(), treePath.getLastPathComponent());
            if (indexOfChild != -1) {
                return this.getBounds(nodeForPath2, indexOfChild, rectangle);
            }
        }
        return null;
    }
    
    @Override
    public TreePath getPathForRow(int n) {
        if (n >= 0 && n < this.getRowCount() && this.root.getPathForRow(n, this.getRowCount(), this.info)) {
            return this.info.getPath();
        }
        return null;
    }
    
    @Override
    public int getRowForPath(TreePath treePath) {
        if (treePath == null || this.root == null) {
            return -1;
        }
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            return nodeForPath.getRow();
        }
        TreePath parentPath = treePath.getParentPath();
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath2 = this.getNodeForPath(parentPath, true, false);
        if (nodeForPath2 != null && nodeForPath2.isExpanded()) {
            return nodeForPath2.getRowToModelIndex(this.treeModel.getIndexOfChild(parentPath.getLastPathComponent(), treePath.getLastPathComponent()));
        }
        return -1;
    }
    
    @Override
    public TreePath getPathClosestTo(int n, int n2) {
        if (this.getRowCount() == 0) {
            return null;
        }
        return this.getPathForRow(this.getRowContainingYLocation(n2));
    }
    
    @Override
    public int getVisibleChildCount(TreePath treePath) {
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath == null) {
            return 0;
        }
        return nodeForPath.getTotalChildCount();
    }
    
    @Override
    public Enumeration<TreePath> getVisiblePathsFrom(TreePath treePath) {
        if (treePath == null) {
            return null;
        }
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        if (nodeForPath != null) {
            return new FixedHeightLayoutCache.VisibleFHTreeStateNodeEnumeration(nodeForPath);
        }
        TreePath parentPath = treePath.getParentPath();
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath2 = this.getNodeForPath(parentPath, true, false);
        if (nodeForPath2 != null && nodeForPath2.isExpanded()) {
            return new FixedHeightLayoutCache.VisibleFHTreeStateNodeEnumeration(nodeForPath2, this.treeModel.getIndexOfChild(parentPath.getLastPathComponent(), treePath.getLastPathComponent()));
        }
        return null;
    }
    
    @Override
    public void setExpandedState(TreePath treePath, boolean b) {
        if (b) {
            this.ensurePathIsExpanded(treePath, true);
        }
        else if (treePath != null) {
            TreePath parentPath = treePath.getParentPath();
            if (parentPath != null) {
                FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(parentPath, false, true);
                if (nodeForPath != null) {
                    nodeForPath.makeVisible();
                }
            }
            FixedHeightLayoutCache.FHTreeStateNode nodeForPath2 = this.getNodeForPath(treePath, true, false);
            if (nodeForPath2 != null) {
                nodeForPath2.collapse(true);
            }
        }
    }
    
    @Override
    public boolean getExpandedState(TreePath treePath) {
        FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, true, false);
        return nodeForPath != null && (nodeForPath.isVisible() && nodeForPath.isExpanded());
    }
    
    @Override
    public void treeNodesChanged(TreeModelEvent event) {
        if (event != null) {
            FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(SwingUtilities2.getTreePath(event, this.getModel()), false, false);
            int[] childIndices = event.getChildIndices();
            if (nodeForPath != null) {
                int length;
                if (childIndices != null && (length = childIndices.length) > 0) {
                    Object userObject = nodeForPath.getUserObject();
                    for (int i = 0; i < length; ++i) {
                        FixedHeightLayoutCache.FHTreeStateNode childAtModelIndex = nodeForPath.getChildAtModelIndex(childIndices[i]);
                        if (childAtModelIndex != null) {
                            childAtModelIndex.setUserObject(this.treeModel.getChild(userObject, childIndices[i]));
                        }
                    }
                    if (nodeForPath.isVisible() && nodeForPath.isExpanded()) {
                        this.visibleNodesChanged();
                    }
                }
                else if (nodeForPath == this.root && nodeForPath.isVisible() && nodeForPath.isExpanded()) {
                    this.visibleNodesChanged();
                }
            }
        }
    }
    
    @Override
    public void treeNodesInserted(TreeModelEvent event) {
        if (event != null) {
            FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(SwingUtilities2.getTreePath(event, this.getModel()), false, false);
            int[] childIndices = event.getChildIndices();
            int length;
            if (nodeForPath != null && childIndices != null && (length = childIndices.length) > 0) {
                int n = nodeForPath.isVisible() && nodeForPath.isExpanded();
                for (int i = 0; i < length; ++i) {
                    nodeForPath.childInsertedAtModelIndex(childIndices[i], (boolean)n);
                }
                if (n && this.treeSelectionModel != null) {
                    this.treeSelectionModel.resetRowSelection();
                }
                if (nodeForPath.isVisible()) {
                    this.visibleNodesChanged();
                }
            }
        }
    }
    
    @Override
    public void treeNodesRemoved(TreeModelEvent event) {
        if (event != null) {
            FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(SwingUtilities2.getTreePath(event, this.getModel()), false, false);
            int[] childIndices = event.getChildIndices();
            int length;
            if (nodeForPath != null && childIndices != null && (length = childIndices.length) > 0) {
                event.getChildren();
                int n = nodeForPath.isVisible() && nodeForPath.isExpanded();
                for (int i = length - 1; i >= 0; --i) {
                    nodeForPath.removeChildAtModelIndex(childIndices[i], (boolean)n);
                }
                if (!(n == 0)) {
                    if (this.treeSelectionModel != null) {
                        this.treeSelectionModel.resetRowSelection();
                    }
                    if (this.treeModel.getChildCount(nodeForPath.getUserObject()) == 0 && nodeForPath.isLeaf()) {
                        nodeForPath.collapse(false);
                    }
                    this.visibleNodesChanged();
                }
                else if (nodeForPath.isVisible()) {
                    this.visibleNodesChanged();
                }
            }
        }
    }
    
    @Override
    public void treeStructureChanged(TreeModelEvent event) {
        if (event != null) {
            TreePath treePath = SwingUtilities2.getTreePath(event, this.getModel());
            FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(treePath, false, false);
            if (nodeForPath == this.root || (nodeForPath == null && ((treePath == null && this.treeModel != null && this.treeModel.getRoot() == null) || (treePath != null && treePath.getPathCount() <= 1)))) {
                this.rebuild(true);
            }
            else if (nodeForPath != null) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)nodeForPath.getParent();
                int expanded = nodeForPath.isExpanded();
                int visible = nodeForPath.isVisible();
                int index = fhTreeStateNode.getIndex(nodeForPath);
                nodeForPath.collapse(false);
                fhTreeStateNode.remove(index);
                if (visible && expanded) {
                    fhTreeStateNode.resetChildrenRowsFrom(nodeForPath.getRow(), index, nodeForPath.getChildIndex());
                    this.getNodeForPath(treePath, false, true).expand();
                }
                if (this.treeSelectionModel != null && visible && expanded) {
                    this.treeSelectionModel.resetRowSelection();
                }
                if (!(visible == 0)) {
                    this.visibleNodesChanged();
                }
            }
        }
    }
    
    private void visibleNodesChanged() {
    }
    
    private Rectangle getBounds(FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode, int n, Rectangle rectangle) {
        int n2;
        Object o;
        int expanded;
        int level;
        if (n == -1) {
            n2 = fhTreeStateNode.getRow();
            o = fhTreeStateNode.getUserObject();
            expanded = fhTreeStateNode.isExpanded();
            level = fhTreeStateNode.getLevel();
        }
        else {
            n2 = fhTreeStateNode.getRowToModelIndex(n);
            o = this.treeModel.getChild(fhTreeStateNode.getUserObject(), n);
            expanded = false;
            level = fhTreeStateNode.getLevel() + 1;
        }
        Rectangle nodeDimensions = this.getNodeDimensions(o, n2, level, (boolean)expanded, this.boundsBuffer);
        if (nodeDimensions == null) {
            return null;
        }
        if (rectangle == null) {
            rectangle = new Rectangle();
        }
        rectangle.x = nodeDimensions.x;
        rectangle.height = this.getRowHeight();
        rectangle.y = n2 * rectangle.height;
        rectangle.width = nodeDimensions.width;
        return rectangle;
    }
    
    private void adjustRowCountBy(int n) {
        this.rowCount += n;
    }
    
    private void addMapping(FixedHeightLayoutCache.FHTreeStateNode value) {
        this.treePathMapping.put(value.getTreePath(), value);
    }
    
    private void removeMapping(FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode) {
        this.treePathMapping.remove(fhTreeStateNode.getTreePath());
    }
    
    private FixedHeightLayoutCache.FHTreeStateNode getMapping(TreePath key) {
        return (FixedHeightLayoutCache.FHTreeStateNode)this.treePathMapping.get(key);
    }
    
    private void rebuild(boolean b) {
        this.treePathMapping.clear();
        Object root;
        if (this.treeModel != null && (root = this.treeModel.getRoot()) != null) {
            this.root = this.createNodeForValue(root, 0);
            this.root.path = new TreePath(root);
            this.addMapping(this.root);
            if (this.isRootVisible()) {
                this.rowCount = 1;
                this.root.row = 0;
            }
            else {
                this.rowCount = 0;
                this.root.row = -1;
            }
            this.root.expand();
        }
        else {
            this.root = null;
            this.rowCount = 0;
        }
        if (b && this.treeSelectionModel != null) {
            this.treeSelectionModel.clearSelection();
        }
        this.visibleNodesChanged();
    }
    
    private int getRowContainingYLocation(int n) {
        if (this.getRowCount() == 0) {
            return -1;
        }
        return Math.max(0, Math.min(this.getRowCount() - 1, n / this.getRowHeight()));
    }
    
    private boolean ensurePathIsExpanded(TreePath parentPath, boolean b) {
        if (parentPath != null) {
            if (this.treeModel.isLeaf(parentPath.getLastPathComponent())) {
                parentPath = parentPath.getParentPath();
                b = true;
            }
            if (parentPath != null) {
                FixedHeightLayoutCache.FHTreeStateNode nodeForPath = this.getNodeForPath(parentPath, false, true);
                if (nodeForPath != null) {
                    nodeForPath.makeVisible();
                    if (b) {
                        nodeForPath.expand();
                    }
                    return true;
                }
            }
        }
        return false;
    }
    
    private FixedHeightLayoutCache.FHTreeStateNode createNodeForValue(Object o, int n) {
        return new FixedHeightLayoutCache.FHTreeStateNode(o, n, -1);
    }
    
    private FixedHeightLayoutCache.FHTreeStateNode getNodeForPath(TreePath treePath, boolean b, boolean b2) {
        if (treePath != null) {
            FixedHeightLayoutCache.FHTreeStateNode mapping = this.getMapping(treePath);
            if (mapping != null) {
                if (b && !mapping.isVisible()) {
                    return null;
                }
                return mapping;
            }
            else {
                if (b) {
                    return null;
                }
                Stack<TreePath> item;
                if (this.tempStacks.size() == 0) {
                    item = new Stack();
                }
                else {
                    item = (Stack)this.tempStacks.pop();
                }
                try {
                    item.push(treePath);
                    treePath = treePath.getParentPath();
                    while (treePath != null) {
                        FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = this.getMapping(treePath);
                        if (fhTreeStateNode != null) {
                            while (fhTreeStateNode != null && item.size() > 0) {
                                treePath = (TreePath)item.pop();
                                fhTreeStateNode = fhTreeStateNode.createChildFor(treePath.getLastPathComponent());
                            }
                            return fhTreeStateNode;
                        }
                        item.push(treePath);
                        treePath = treePath.getParentPath();
                    }
                }
                finally {
                    item.removeAllElements();
                    this.tempStacks.push(item);
                }
                return null;
            }
        }
        else {
            return null;
        }
    }
    
    static void access$100(FixedHeightLayoutCache fixedHeightLayoutCache, FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode) {
        fixedHeightLayoutCache.addMapping(fhTreeStateNode);
    }
    
    static void access$200(FixedHeightLayoutCache fixedHeightLayoutCache, FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode) {
        fixedHeightLayoutCache.removeMapping(fhTreeStateNode);
    }
    
    static FixedHeightLayoutCache.FHTreeStateNode access$300(FixedHeightLayoutCache fixedHeightLayoutCache, Object o, int n) {
        return fixedHeightLayoutCache.createNodeForValue(o, n);
    }
    
    static void access$400(FixedHeightLayoutCache fixedHeightLayoutCache, int n) {
        fixedHeightLayoutCache.adjustRowCountBy(n);
    }
    
    private class FHTreeStateNode extends DefaultMutableTreeNode
    {
        protected boolean isExpanded;
        protected int childIndex;
        protected int childCount;
        protected int row;
        protected TreePath path;
        
        public FHTreeStateNode(Object o, int childIndex, int row) {
            super(o);
            this.childIndex = childIndex;
            this.row = row;
        }
        
        @Override
        public void setParent(MutableTreeNode parent) {
            super.setParent(parent);
            if (parent != null) {
                this.path = ((FixedHeightLayoutCache.FHTreeStateNode)parent).getTreePath().pathByAddingChild(this.getUserObject());
                FixedHeightLayoutCache.access$100(FixedHeightLayoutCache.this, this);
            }
        }
        
        @Override
        public void remove(int n) {
            ((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(n)).removeFromMapping();
            super.remove(n);
        }
        
        @Override
        public void setUserObject(Object userObject) {
            super.setUserObject(userObject);
            if (this.path != null) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
                if (fhTreeStateNode != null) {
                    this.resetChildrenPaths(fhTreeStateNode.getTreePath());
                }
                else {
                    this.resetChildrenPaths(null);
                }
            }
        }
        
        public int getChildIndex() {
            return this.childIndex;
        }
        
        public TreePath getTreePath() {
            return this.path;
        }
        
        public FixedHeightLayoutCache.FHTreeStateNode getChildAtModelIndex(int n) {
            int i = this.getChildCount() - 1;
            while (i >= 0) {
                if (((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).childIndex == n) {
                    return (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                }
                --i;
            }
            return null;
        }
        
        public boolean isVisible() {
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
            return fhTreeStateNode == null || (fhTreeStateNode.isExpanded() && fhTreeStateNode.isVisible());
        }
        
        public int getRow() {
            return this.row;
        }
        
        public int getRowToModelIndex(int n) {
            int n2 = this.getRow() + 1;
            int i = 0;
            while (i < this.getChildCount()) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                if (fhTreeStateNode.childIndex >= n) {
                    if (fhTreeStateNode.childIndex == n) {
                        return fhTreeStateNode.row;
                    }
                    if (i == 0) {
                        return this.getRow() + 1 + n;
                    }
                    return fhTreeStateNode.row - (fhTreeStateNode.childIndex - n);
                }
                else {
                    ++i;
                }
            }
            return this.getRow() + 1 + this.getTotalChildCount() - (this.childCount - n);
        }
        
        public int getTotalChildCount() {
            if (this.isExpanded()) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
                int index;
                if (fhTreeStateNode != null && (index = fhTreeStateNode.getIndex(this)) + 1 < fhTreeStateNode.getChildCount()) {
                    FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode2 = (FixedHeightLayoutCache.FHTreeStateNode)fhTreeStateNode.getChildAt(index + 1);
                    return fhTreeStateNode2.row - this.row - (fhTreeStateNode2.childIndex - this.childIndex);
                }
                int childCount = this.childCount;
                for (int i = this.getChildCount() - 1; i >= 0; --i) {
                    childCount += ((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).getTotalChildCount();
                }
                return childCount;
            }
            else {
                return 0;
            }
        }
        
        public boolean isExpanded() {
            return this.isExpanded;
        }
        
        public int getVisibleLevel() {
            if (FixedHeightLayoutCache.this.isRootVisible()) {
                return this.getLevel();
            }
            return this.getLevel() - 1;
        }
        
        protected void resetChildrenPaths(TreePath treePath) {
            FixedHeightLayoutCache.access$200(FixedHeightLayoutCache.this, this);
            if (treePath == null) {
                this.path = new TreePath(this.getUserObject());
            }
            else {
                this.path = treePath.pathByAddingChild(this.getUserObject());
            }
            FixedHeightLayoutCache.access$100(FixedHeightLayoutCache.this, this);
            for (int i = this.getChildCount() - 1; i >= 0; --i) {
                ((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).resetChildrenPaths(this.path);
            }
        }
        
        protected void removeFromMapping() {
            if (this.path != null) {
                FixedHeightLayoutCache.access$200(FixedHeightLayoutCache.this, this);
                for (int i = this.getChildCount() - 1; i >= 0; --i) {
                    ((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).removeFromMapping();
                }
            }
        }
        
        protected FixedHeightLayoutCache.FHTreeStateNode createChildFor(Object o) {
            int indexOfChild = FixedHeightLayoutCache.this.treeModel.getIndexOfChild(this.getUserObject(), o);
            if (indexOfChild < 0) {
                return null;
            }
            FixedHeightLayoutCache.FHTreeStateNode access$300 = FixedHeightLayoutCache.access$300(FixedHeightLayoutCache.this, o, indexOfChild);
            int rowToModelIndex;
            if (this.isVisible()) {
                rowToModelIndex = this.getRowToModelIndex(indexOfChild);
            }
            else {
                rowToModelIndex = -1;
            }
            access$300.row = rowToModelIndex;
            int i = 0;
            while (i < this.getChildCount()) {
                if (((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).childIndex > indexOfChild) {
                    this.insert(access$300, i);
                    return access$300;
                }
                ++i;
            }
            this.add(access$300);
            return access$300;
        }
        
        protected void adjustRowBy(int n) {
            this.row += n;
            if (this.isExpanded) {
                for (int i = this.getChildCount() - 1; i >= 0; --i) {
                    ((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).adjustRowBy(n);
                }
            }
        }
        
        protected void adjustRowBy(int n, int n2) {
            if (this.isExpanded) {
                for (int i = this.getChildCount() - 1; i >= n2; --i) {
                    ((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).adjustRowBy(n);
                }
            }
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
            if (fhTreeStateNode != null) {
                fhTreeStateNode.adjustRowBy(n, fhTreeStateNode.getIndex(this) + 1);
            }
        }
        
        protected void didExpand() {
            int setRowAndChildren = this.setRowAndChildren(this.row);
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
            int n = setRowAndChildren - this.row - 1;
            if (fhTreeStateNode != null) {
                fhTreeStateNode.adjustRowBy(n, fhTreeStateNode.getIndex(this) + 1);
            }
            FixedHeightLayoutCache.access$400(FixedHeightLayoutCache.this, n);
        }
        
        protected int setRowAndChildren(int row) {
            this.row = row;
            if (!this.isExpanded()) {
                return this.row + 1;
            }
            int setRowAndChildren = this.row + 1;
            int n = 0;
            for (int childCount = this.getChildCount(), i = 0; i < childCount; ++i) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                setRowAndChildren += fhTreeStateNode.childIndex - n;
                n = fhTreeStateNode.childIndex + 1;
                if (fhTreeStateNode.isExpanded) {
                    setRowAndChildren = fhTreeStateNode.setRowAndChildren(setRowAndChildren);
                }
                else {
                    fhTreeStateNode.row = setRowAndChildren++;
                }
            }
            return setRowAndChildren + this.childCount - n;
        }
        
        protected void resetChildrenRowsFrom(int setRowAndChildren, int i, int n) {
            int childCount;
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode;
            for (childCount = this.getChildCount(), ; i < childCount; ++i) {
                fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                setRowAndChildren += fhTreeStateNode.childIndex - n;
                n = fhTreeStateNode.childIndex + 1;
                if (fhTreeStateNode.isExpanded) {
                    setRowAndChildren = fhTreeStateNode.setRowAndChildren(setRowAndChildren);
                }
                else {
                    fhTreeStateNode.row = setRowAndChildren++;
                }
            }
            int n2 = setRowAndChildren + (this.childCount - n);
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode2 = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
            if (fhTreeStateNode2 != null) {
                fhTreeStateNode2.resetChildrenRowsFrom(n2, fhTreeStateNode2.getIndex(this) + 1, this.childIndex + 1);
            }
            else {
                FixedHeightLayoutCache.this.rowCount = n2;
            }
        }
        
        protected void makeVisible() {
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
            if (fhTreeStateNode != null) {
                fhTreeStateNode.expandParentAndReceiver();
            }
        }
        
        protected void expandParentAndReceiver() {
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getParent();
            if (fhTreeStateNode != null) {
                fhTreeStateNode.expandParentAndReceiver();
            }
            this.expand();
        }
        
        protected void expand() {
            if (!this.isExpanded && !this.isLeaf()) {
                int visible = this.isVisible();
                this.isExpanded = true;
                this.childCount = FixedHeightLayoutCache.this.treeModel.getChildCount(this.getUserObject());
                if (!(visible == 0)) {
                    this.didExpand();
                }
                if (visible && FixedHeightLayoutCache.this.treeSelectionModel != null) {
                    FixedHeightLayoutCache.this.treeSelectionModel.resetRowSelection();
                }
            }
        }
        
        protected void collapse(boolean b) {
            if (this.isExpanded) {
                if (this.isVisible() && b) {
                    int totalChildCount = this.getTotalChildCount();
                    this.isExpanded = false;
                    FixedHeightLayoutCache.access$400(FixedHeightLayoutCache.this, -totalChildCount);
                    this.adjustRowBy(-totalChildCount, 0);
                }
                else {
                    this.isExpanded = false;
                }
                if (b && this.isVisible() && FixedHeightLayoutCache.this.treeSelectionModel != null) {
                    FixedHeightLayoutCache.this.treeSelectionModel.resetRowSelection();
                }
            }
        }
        
        @Override
        public boolean isLeaf() {
            TreeModel model = FixedHeightLayoutCache.this.getModel();
            return model == null || model.isLeaf(this.getUserObject());
        }
        
        protected void addNode(FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode) {
            boolean n = false;
            int childIndex = fhTreeStateNode.getChildIndex();
            for (int i = 0, childCount = this.getChildCount(); i < childCount; ++i) {
                if (((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).getChildIndex() > childIndex) {
                    n = true;
                    this.insert(fhTreeStateNode, i);
                    i = childCount;
                }
            }
            if (n == 0) {
                this.add(fhTreeStateNode);
            }
        }
        
        protected void removeChildAtModelIndex(int n, boolean b) {
            FixedHeightLayoutCache.FHTreeStateNode childAtModelIndex = this.getChildAtModelIndex(n);
            if (childAtModelIndex != null) {
                int row = childAtModelIndex.getRow();
                int index = this.getIndex(childAtModelIndex);
                childAtModelIndex.collapse(false);
                this.remove(index);
                this.adjustChildIndexs(index, -1);
                --this.childCount;
                if (b) {
                    this.resetChildrenRowsFrom(row, index, n);
                }
            }
            else {
                int childCount = this.getChildCount();
                int i = 0;
                while (i < childCount) {
                    if (((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).childIndex >= n) {
                        if (b) {
                            this.adjustRowBy(-1, i);
                            FixedHeightLayoutCache.access$400(FixedHeightLayoutCache.this, -1);
                        }
                        while (i < childCount) {
                            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                            --fhTreeStateNode.childIndex;
                            ++i;
                        }
                        --this.childCount;
                        return;
                    }
                    else {
                        ++i;
                    }
                }
                if (b) {
                    this.adjustRowBy(-1, childCount);
                    FixedHeightLayoutCache.access$400(FixedHeightLayoutCache.this, -1);
                }
                --this.childCount;
            }
        }
        
        protected void adjustChildIndexs(int i, int n) {
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode;
            for (; i < this.getChildCount(); ++i) {
                fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                fhTreeStateNode.childIndex += n;
            }
        }
        
        protected void childInsertedAtModelIndex(int n, boolean b) {
            int childCount = this.getChildCount();
            int i = 0;
            while (i < childCount) {
                if (((FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i)).childIndex >= n) {
                    if (b) {
                        this.adjustRowBy(1, i);
                        FixedHeightLayoutCache.access$400(FixedHeightLayoutCache.this, 1);
                    }
                    while (i < childCount) {
                        FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                        ++fhTreeStateNode.childIndex;
                        ++i;
                    }
                    ++this.childCount;
                    return;
                }
                else {
                    ++i;
                }
            }
            if (b) {
                this.adjustRowBy(1, childCount);
                FixedHeightLayoutCache.access$400(FixedHeightLayoutCache.this, 1);
            }
            ++this.childCount;
        }
        
        protected boolean getPathForRow(int n, int n2, FixedHeightLayoutCache.SearchInfo searchInfo) {
            if (this.row == n) {
                searchInfo.node = this;
                searchInfo.isNodeParentNode = false;
                searchInfo.childIndex = this.childIndex;
                return true;
            }
            FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = null;
            int i = 0;
            while (i < this.getChildCount()) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode2 = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                if (fhTreeStateNode2.row > n) {
                    if (i == 0) {
                        searchInfo.node = this;
                        searchInfo.isNodeParentNode = true;
                        searchInfo.childIndex = n - this.row - 1;
                        return true;
                    }
                    int n3 = 1 + fhTreeStateNode2.row - (fhTreeStateNode2.childIndex - fhTreeStateNode.childIndex);
                    if (n < n3) {
                        return fhTreeStateNode.getPathForRow(n, n3, searchInfo);
                    }
                    searchInfo.node = this;
                    searchInfo.isNodeParentNode = true;
                    searchInfo.childIndex = n - n3 + fhTreeStateNode.childIndex + 1;
                    return true;
                }
                else {
                    fhTreeStateNode = fhTreeStateNode2;
                    ++i;
                }
            }
            if (fhTreeStateNode != null) {
                int n4 = n2 - (this.childCount - fhTreeStateNode.childIndex) + 1;
                if (n < n4) {
                    return fhTreeStateNode.getPathForRow(n, n4, searchInfo);
                }
                searchInfo.node = this;
                searchInfo.isNodeParentNode = true;
                searchInfo.childIndex = n - n4 + fhTreeStateNode.childIndex + 1;
                return true;
            }
            else {
                int childIndex = n - this.row - 1;
                if (childIndex >= this.childCount) {
                    return false;
                }
                searchInfo.node = this;
                searchInfo.isNodeParentNode = true;
                searchInfo.childIndex = childIndex;
                return true;
            }
        }
        
        protected int getCountTo(int n) {
            int n2 = n + 1;
            for (int i = 0, childCount = this.getChildCount(); i < childCount; ++i) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                if (fhTreeStateNode.childIndex >= n) {
                    i = childCount;
                }
                else {
                    n2 += fhTreeStateNode.getTotalChildCount();
                }
            }
            if (this.parent != null) {
                return n2 + ((FixedHeightLayoutCache.FHTreeStateNode)this.getParent()).getCountTo(this.childIndex);
            }
            if (!FixedHeightLayoutCache.this.isRootVisible()) {
                return n2 - 1;
            }
            return n2;
        }
        
        protected int getNumExpandedChildrenTo(int n) {
            int i = 0;
            while (i < this.getChildCount()) {
                FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode = (FixedHeightLayoutCache.FHTreeStateNode)this.getChildAt(i);
                if (fhTreeStateNode.childIndex >= n) {
                    return n;
                }
                n += fhTreeStateNode.getTotalChildCount();
                ++i;
            }
            return n;
        }
        
        protected void didAdjustTree() {
        }
    }
    
    private class SearchInfo
    {
        protected FixedHeightLayoutCache.FHTreeStateNode node;
        protected boolean isNodeParentNode;
        protected int childIndex;
        
        protected TreePath getPath() {
            if (this.node == null) {
                return null;
            }
            if (this.isNodeParentNode) {
                return this.node.getTreePath().pathByAddingChild(FixedHeightLayoutCache.this.treeModel.getChild(this.node.getUserObject(), this.childIndex));
            }
            return this.node.path;
        }
    }
    
    private class VisibleFHTreeStateNodeEnumeration implements Enumeration<TreePath>
    {
        protected FixedHeightLayoutCache.FHTreeStateNode parent;
        protected int nextIndex;
        protected int childCount;
        
        protected VisibleFHTreeStateNodeEnumeration(FixedHeightLayoutCache fixedHeightLayoutCache, FixedHeightLayoutCache.FHTreeStateNode fhTreeStateNode) {
            this(fixedHeightLayoutCache, fhTreeStateNode, -1);
        }
        
        protected VisibleFHTreeStateNodeEnumeration(FixedHeightLayoutCache.FHTreeStateNode parent, int nextIndex) {
            this.parent = parent;
            this.nextIndex = nextIndex;
            this.childCount = FixedHeightLayoutCache.this.treeModel.getChildCount(this.parent.getUserObject());
        }
        
        @Override
        public boolean hasMoreElements() {
            return this.parent != null;
        }
        
        @Override
        public TreePath nextElement() {
            if (!this.hasMoreElements()) {
                throw new NoSuchElementException("No more visible paths");
            }
            TreePath treePath;
            if (this.nextIndex == -1) {
                treePath = this.parent.getTreePath();
            }
            else {
                FixedHeightLayoutCache.FHTreeStateNode childAtModelIndex = this.parent.getChildAtModelIndex(this.nextIndex);
                if (childAtModelIndex == null) {
                    treePath = this.parent.getTreePath().pathByAddingChild(FixedHeightLayoutCache.this.treeModel.getChild(this.parent.getUserObject(), this.nextIndex));
                }
                else {
                    treePath = childAtModelIndex.getTreePath();
                }
            }
            this.updateNextObject();
            return treePath;
        }
        
        protected void updateNextObject() {
            if (!this.updateNextIndex()) {
                this.findNextValidParent();
            }
        }
        
        protected boolean findNextValidParent() {
            if (this.parent == FixedHeightLayoutCache.this.root) {
                this.parent = null;
                return false;
            }
            while (this.parent != null) {
                FixedHeightLayoutCache.FHTreeStateNode parent = (FixedHeightLayoutCache.FHTreeStateNode)this.parent.getParent();
                if (parent != null) {
                    this.nextIndex = this.parent.childIndex;
                    this.parent = parent;
                    this.childCount = FixedHeightLayoutCache.this.treeModel.getChildCount(this.parent.getUserObject());
                    if (this.updateNextIndex()) {
                        return true;
                    }
                    continue;
                }
                else {
                    this.parent = null;
                }
            }
            return false;
        }
        
        protected boolean updateNextIndex() {
            if (this.nextIndex == -1 && !this.parent.isExpanded()) {
                return false;
            }
            if (this.childCount == 0) {
                return false;
            }
            if (++this.nextIndex >= this.childCount) {
                return false;
            }
            FixedHeightLayoutCache.FHTreeStateNode childAtModelIndex = this.parent.getChildAtModelIndex(this.nextIndex);
            if (childAtModelIndex != null && childAtModelIndex.isExpanded()) {
                this.parent = childAtModelIndex;
                this.nextIndex = -1;
                this.childCount = FixedHeightLayoutCache.this.treeModel.getChildCount(childAtModelIndex.getUserObject());
            }
            return true;
        }
    }
}
